/* AbstractTrackFitter.java

    Defines standard variables and provides default implementation of accessor methods.

 Created by  Mike Ronan  May '01
 Modified by

 */

package hep.lcd.recon.ztracking;

import hep.lcd.event.Track;

import Jama.*;


public abstract class AbstractTrackFitter implements TrackFitter
{
    protected String Fitter = "Undefined";

    protected Track track;
    protected TrackCandidate candidate;
    protected TrackHotList hotList;
    protected TrackParameters parameters;

    protected boolean useVXD = true;
    protected boolean useTracker = true;
    protected boolean useBP=true;

    protected Matrix erma;
    protected boolean success = false;
    protected double chi2 = 0.;
    protected double NDoF = 0.;

    protected boolean badTrack = false;
    protected double veryLarge = 10000000000.;
 

    public AbstractTrackFitter()
    {
	this(false);
    }
    /** Sets up track fitter */
    public AbstractTrackFitter(boolean hist)
    {
	if (trace||debug) {
	  String text = ClassName+"  version 1.1.8";
	}

	if (!hist) setNoHist();
	else setHist();
    }
    /** Sets Track fitter histogram options */
    protected void setHist()  { Hist = Hstd || (Htrks||Hhits||Hpnts); }

    /** Turns on Track fitter production histograms. */
    public void setHprod() { Hstd = true; setHist(); }
    /** Turns on Track fitter test histograms. */
    public void setHtest() { setHprod(); setHtrks(); setHhits(); setHpnts(); }
    /** Turns on Track fitter development histograms. */
    public void setHdev() { setHprod(); setHtrks(); }

    /** Turns on Track fitter track histograms:  */
    public void setHtrks() { Htrks = true; HEvents = 0; setHist(); }
    /** Turns on Track fitter hit histograms:  */
    public void setHhits() { Hhits = true; HEvents = 0; setHist(); }
    /** Turns on Track fitter point histograms: point assignment  */
    public void setHpnts() { Hpnts = true; HEvents = 0; setHist(); }

    /** Turns off all histogram packages */
    public void setNoHist() { Hist = Hstd = Htrks = Hhits = Hpnts = false; }



    /** Setup Fitter parameters and initiate callback to set fitting strategy. */
    public void setupFitter(Tracker tracker, VertexDetector vxd)
    {
	if (debug) System.out.println(ClassName+"::setupFitter: Set vertex detector to "+vxd.getType()+
				      " and tracker to "+tracker.getType());
	this.tracker = tracker;
	this.vxd = vxd;

	// Set default histogram folder
	defaultFolder = "/Tracking/Fitting"+"/"+Fitter;

	// Setup instance parameters which completes in a callback to setup fitting strategy..
	setupParameters();
    }

    public abstract void setupParameters();

    public void setVXD(VertexDetector vxd)
    {
	this.vxd = vxd;
    }
    public void setTracker(Tracker tracker)
    {
	this.tracker = tracker;
    }
    public void useVXD(boolean use) { useVXD = use; }
    public void useTracker(boolean use) { useTracker = use; }


    /** Returns status of track fitting. */
    public boolean fitSuccess()
    {
	return success;
    }
    /** Returns fitted track parameters. */
    public TrackParameters getFittedParameters()
    {
	return parameters;
    }
    /** Get overall chi-squared for the track. */
    public double getChi2()
    {
	if (!badTrack) return chi2;
	else return veryLarge;
    }
    /** Get number of degrees of freedom for the track. */
    public double getNDF()
    {
	return NDoF;
    }
    /** Get track error matrix. */
    public double[][] getErrorMatrix()
    {
	return erma.getArray();
    }


    String ClassName = "AbstractTrackFitter::"; protected String text;

    // Inline debugging and analysis
    protected boolean debug = true;
    protected boolean trace = true;  // Set for testing new versions

    // Printing
    protected boolean first = true;

    // Histogramming
    protected String defaultFolder;
    protected int HEvents = 0, MaxHistEvts = 1000, MaxTrkHistEvts = 100, MaxHitHistEvts = 10, MaxPntHistEvts = 10;
    protected boolean Hist;
    protected boolean Hstd = false;
    protected boolean Htrks = false;
    protected boolean Hhits = false;
    protected boolean Hpnts = false;


    protected Tracker tracker; 
    protected VertexDetector vxd;
}
