/* RadiationLengths.java
 *
 *  This class provides access to radiation lengths table for different materials.
 *  Its data is copied from PDG "Review of Particle Physics", 1998
 *
 */

package hep.lcd.recon.ztracking;

import java.util.Hashtable;


/** Provides access to radiation lengths table for different materials.
 * Its data are copied from PDG "Review of Particle Physics", 1998.
 */
public class RadiationLengths extends Hashtable
{

    // Initialize table
    public RadiationLengths()
    {
	this(64);
	this.put("air",new Double(30423.));
	this.put("al",new Double(8.9));
	this.put("be",new Double(35.28));
	this.put("carbon",new Double(18.895));
	this.put("cu",new Double(1.43));
	this.put("fe",new Double(1.76));
	this.put("stainless_steel",new Double(1.766));
	this.put("g10",new Double(19.4));
	this.put("pb",new Double(0.56));
	this.put("polystyrene",new Double(42.4));
	this.put("polystyrenefoam",new Double(683.7));
	this.put("si",new Double(9.36));
	this.put("w",new Double(0.35));
	this.put("ti",new Double(3.563));
	this.put("u",new Double(0.32));
	this.put("nitrogen_liq",new Double(47.1));
	this.put("nitrogen_gas",new Double(30392.));
	this.put("argon_liq",new Double(14.));
	this.put("argon_gas",new Double(10970.));
	this.put("glass",new Double(12.7));
	this.put("quartz",new Double(12.3));
	this.put("nylon",new Double(36.7));
	this.put("lexan",new Double(34.6));
	this.put("mylar",new Double(28.7));
	this.put("kapton",new Double(28.6));
	this.put("polyethylene",new Double(47.9));
	this.put("plexiglas",new Double(34.4));
	this.put("teflon",new Double(15.8));
	this.put("tyvek",new Double(44.78));
	this.put("baf2",new Double(2.05));
	this.put("bgo",new Double(1.12));
	this.put("csi",new Double(1.85));
	this.put("lif",new Double(14.91));
	this.put("naf",new Double(11.68));
	this.put("nai",new Double(2.59));
	this.put("silica",new Double(150.));  
    }

    public RadiationLengths(int ne)
    {
	super(ne);
    } 

    /**
     * Return radiation length in cm
     * @param material - name of material 
     */
    public synchronized double getRL(String material)
    {
	String lkey = material.toLowerCase();
	return ((Double) this.get(lkey)).doubleValue();
    }
}
