/* TrackCandidate.java


 Created by Mike Ronan  Dec '98
 Modified by

 */

package hep.lcd.recon.ztracking;

import hep.lcd.event.*;

import hep.analysis.*;

import java.text.*;
import java.util.*;


public class TrackCandidate
{
    boolean Candidate=true;

    int trkNum;
    TrackHotList hits;
    TrackParameters parameters;
    TrackExtrapolations extrapolations;
    double   ChiSq=0.;
    int      npteff=0;

    MCParticle mcp;
    double mcpFraction;
    double mcpTrackerFraction;


    public TrackCandidate(int it, TrackHotList hits, double[] trackPar)
    {
	this(it,hits,trackPar,true);
    }
    public TrackCandidate(int it, TrackHotList hits, double[] trackPar, boolean hist)
    {
	trkNum = it;
	this.hits = hits;

	double d0 = trackPar[0];
	double phi0 = trackPar[1];
	double curv = trackPar[2];
	double z0 = trackPar[3];
	double tandip = trackPar[4];

	parameters = new TrackParameters(d0,phi0,curv,z0,tandip);

	determineMCOrigin(hits);

        double rout = 200.;
        double zout = 500.;
	swimmer = new HelicalSwimmer(parameters,rout,zout);

	if (firstTrkCand) {
	    print();
	    firstTrkCand = false;
	}

	if (hist) { Hist = Hstd = Htrks = Hhits = Hpnts = Hdev = true; setHist(); }
    }
    protected void setHist() { Hist = Hist || (Htrks || Hhits || Hpnts || HchiSq || Hdiff) || Hdev; }
    public void setNoHist() { Hist = Hstd = Htrks = Hhits = Hpnts = HchiSq = Hdiff = Hdev = false; }

    public boolean    isCandidate()  { return Candidate; }
    public int        getTrackNum()    { return trkNum; }
    public TrackHotList getHotList()   { return hits; }
    public TrackParameters  getTrackParameters() { return parameters; }
    public TrackExtrapolations  getTrackExtrapolations() { return extrapolations; }
    public double get_d0()     { return parameters.get_d0(); }
    public double get_phi0()   { return parameters.get_phi0(); }
    public double get_omega()  { return parameters.get_omega(); }
    public double get_z0()     { return parameters.get_z0(); }
    public double get_s()      { return parameters.get_s(); }

    public double get_curv()   { return parameters.get_curv(); }
    public double get_tandip() { return parameters.get_tandip(); }

    /** Get Monte Carlo particle parent. */
    public MCParticle getMCParticle() { return mcp; }
    /** Get fraction of hits from parent MC particle. */
    public double getMCPFraction() { return mcpFraction; }
    /** Get fraction of Tracker hits from parent MC particle. */
    public double getMCPTrackerFraction() { return mcpTrackerFraction; }


    /** */
    //public TrkHitOnTrk[] getTrackHits()
    public TrkHitOnTrk[] getTrackHits(Specifications specifications)
    {
	double HalfLength = specifications.getHalfLength();
	double rPhiResolution = specifications.getRPhiResolution();
	double zResolution = specifications.getZResolution();

	if (firstRecalTrk) {
	    System.out.println(ClassName+"getTrackHits: ");
	    System.out.println("  rphi,z resolution = "+rPhiResolution+", "+zResolution);
	    System.out.println(" Print first 10 (& last) hits on track: ");
	    printPoints(10);
	}
	if (Hist) {
	    HTracks++;
	}

	int nhits = hits.nhits;
	if (Hhits && HTracks<MaxHistTrks) Histogram.find("nhits").fill(nhits);

	int n = 0;
	// double x=0,xsq=0,y=0,ysq=0,z=0,zsq=0;
	double ph=0.,r=0.,x=0.,y=0.,z=0.;
        double rl = 0;
	double xc[] = new double[3];;
	double phx = 0.;
	double php = 0.;
	double dph = 0.;

	TrkHitOnTrk hot;
	for (int i=0;i<nhits/3;i++) {
	    hot = hits.get(i);
	    if (hot.isActive) {
		rl = Math.sqrt(hot.x*hot.x + hot.y*hot.y);
		swimmer.swim(rl,HalfLength);
		xc = swimmer.getIntersect();
		phx = Math.atan2(xc[1],xc[0]);
		php = Math.atan2(hot.y,hot.x);
		dph = php-phx;
		if (Math.abs(dph) > Math.PI) {
		    if (php < 0.) php+= 2.*Math.PI;
		    if (phx < 0.) phx+= 2.*Math.PI;
		    dph = php-phx;
		}
		n++;
		ph+=dph; r+=rl; z+=(hot.z-xc[2]);
		if (n==9) break;
	    }
	}
	if (n>0) {
	  ph/=n; r/=n; z/=n;  // Equal errors
	} else {
	  System.err.println(" *** No active hits in first third of list. *** ");
	}
        swimmer.swim(r,HalfLength);
        xc = swimmer.getIntersect();
        phx = Math.atan2(xc[1],xc[0]);
        php = phx + ph;
        z+=xc[2];
        x = r * Math.cos(php);
        y = r * Math.sin(php);   

	TrkHitOnTrk inner = new TrkHitOnTrk("Avg",x,y,z);

	n = 0;
	// x=0;xsq=0;y=0;ysq=0;z=0;zsq=0;
	x=0;y=0;z=0;ph=0.;r=0.;
	int nhalf = nhits/2;
	for (int i=0;i<(nhits/6);i++) {
	  for (int j=-i;j<=i;j+=2*i) {
	    hot = hits.get(nhalf+j);
	    if (hot.isActive) {
              rl = Math.sqrt(hot.x*hot.x + hot.y*hot.y);
	      swimmer.swim(rl,HalfLength);
	      xc = swimmer.getIntersect();
	      phx = Math.atan2(xc[1],xc[0]);
	      php = Math.atan2(hot.y,hot.x);
	      dph = php-phx;
	      if (Math.abs(dph) > Math.PI) {
		if (php < 0.) php+= 2.*Math.PI;
		if (phx < 0.) phx+= 2.*Math.PI;
		dph = php-phx;
	      }
	      n++;
	      ph+=dph; r+=rl; z+=(hot.z-xc[2]);
	      if (i==0 || n==9) break;
	    }
	  }
	  if (n==9) break;
	}
	if (n>0) {
	  ph/=n; r/=n; z/=n;  // Equal errors
	} else {
	  System.err.println(" *** No active hits in 2nd third of list. *** ");
	}

        swimmer.swim(r,HalfLength);
        xc = swimmer.getIntersect();
        phx = Math.atan2(xc[1],xc[0]);
        php = phx + ph;
        z+=xc[2];
        x = r * Math.cos(php);
        y = r * Math.sin(php);   


	TrkHitOnTrk middle = new TrkHitOnTrk("Avg",x,y,z);

	n = 0;
	// x=0;xsq=0;y=0;ysq=0;z=0;zsq=0;
	x=0;y=0;z=0;
	for (int i=nhits-1;i>2*(nhits/3);i--) {
	  hot = hits.get(i);
	  if (hot.isActive) {
            rl = Math.sqrt(hot.x*hot.x + hot.y*hot.y);
            swimmer.swim(rl,HalfLength);
            xc = swimmer.getIntersect();
            phx = Math.atan2(xc[1],xc[0]);
            php = Math.atan2(hot.y,hot.x);
            dph = php-phx;
            if (Math.abs(dph) > Math.PI) {
              if(php < 0.) php+= 2.*Math.PI;
              if(phx < 0.) phx+= 2.*Math.PI;
              dph = php-phx;
            }
	    n++;
	    ph+=dph; r+=rl; z+=(hot.z-xc[2]);
	    if (i==0 || n==9) break;
	  }
	}
	if (n>0) {
	  ph/=n; r/=n; z/=n;  // Equal errors
	} else {
	  System.err.println(" *** No active hits in last third of list. *** ");
	}

        swimmer.swim(r,HalfLength);
        xc = swimmer.getIntersect();
        phx = Math.atan2(xc[1],xc[0]);
        php = phx + ph;
        z+=xc[2];
        x = r * Math.cos(php);
        y = r * Math.sin(php);   


	TrkHitOnTrk outer = new TrkHitOnTrk("Avg",x,y,z);

	TrkHitOnTrk[] selection = {inner,middle,outer};
	return selection;
    }

    /** */
    private void determineMCOrigin(TrackHotList hits)
    {
	if (firstTrkCand) System.out.println(ClassName+"determineMCOrigin: ");

        int nhits = hits.nhits;
        int nmcInfo = 0;
        Hashtable hash = new Hashtable();

        for (int i=0; i<nhits; i++) {
	  TrkHitOnTrk hot = hits.get(i);
	  MCParticle mcp = hot.getMCParticle();
          if (mcp!=null) {
	    nmcInfo++;
	    int nmcp = 0; Integer count;
	    Object object = hash.get(mcp);
	    if (object != null) {
		count = (Integer) object; nmcp = count.intValue(); }
	    nmcp++;
	    count = new Integer(nmcp);
	    hash.put(mcp,count);
	  }
        }

        Enumeration mcps = hash.keys();
        MCParticle mcpm = null;
        int nmax = 0; 
        while (mcps.hasMoreElements()) {
          MCParticle mcp = (MCParticle) mcps.nextElement();
	  Integer count = (Integer) hash.get(mcp);
	  int nmcp = 0;
	  if (count != null) nmcp = count.intValue();
	  if (nmcp > nmax) {
            mcpm = mcp;
	    nmax = nmcp;
	  }
        }             
        if (nmax>0) mcp = mcpm;

	// Fraction of all hits
	mcpFraction = 0.;
        if (nmcInfo > 0) {
	  if (nmax==nmcInfo) mcpFraction = 1.;
          else mcpFraction = (double) nmax / (double) nmcInfo;
	}

	// Fraction of Tracker hits
	int n=0,nh=0;
        for (int i=0; i<nhits; i++) {
	  TrkHitOnTrk hot = hits.get(i);
	  if (hot.trkHit()) {
	    nh++;
	    if (hot.getMCParticle()==mcp) n++;
	  }
	}
	mcpTrackerFraction = 0.;
	if (nh>0) {
	  if (n==nh) mcpTrackerFraction = 1.;
	  else mcpTrackerFraction = (double) n/ (double) nh;
	}
    }

    /** */
    public void recalculateTrack(Specifications specifications)
    {
	if (firstRecalTrk) {
	    System.out.println(ClassName+"recalculateTrack: ");
	    //System.out.println("  NLayers = "+specifications.getNLayers());
	    firstRecalTrk = false;
	}

	TrkHitOnTrk[] trackHits = getTrackHits(specifications);
	TrkHitOnTrk inner  = trackHits[0];
	TrkHitOnTrk middle = trackHits[1];
	TrkHitOnTrk outer  = trackHits[2];

	// Get their coordinate differences
	double P[] = new double[4];
	double Q[] = new double[4];
	double R[] = new double[4];
	P[1] = inner.x - middle.x;
	P[2] = middle.x - outer.x;
	P[3] = outer.x - inner.x;
	Q[1] = inner.y - middle.y;
	Q[2] = middle.y - outer.y;
	Q[3] = outer.y - inner.y;
	// and squares of distances between points
	for (int j = 1; j <=3; j++) R[j] = P[j]*P[j]+Q[j]*Q[j];

	//branch out according to which difference is the largest
	double Rc = 0.5*(R[1]+R[2]+R[3]);
	// if (firstRecalTrk) System.out.println(" R[1],R[2],Rc = "+R[1]+","+R[2]+","+Rc);

	int foundBranch = 0;
	int Jd=0, Jm=0, Jt=0;
	if (R[1] > Rc) {
	    // here, inner and middle points are furthest apart
	    foundBranch = 1;
	    d = inner; m = outer; t = middle;
	    Jd = 2; Jm = 1; Jt = 3;
	}
	if (R[2] > Rc) {
	    // here, outer and middle are furthest
	    foundBranch = 2;
	    d = middle; m = inner; t = outer;
	    Jd = 3; Jm = 2; Jt = 1;
	}
	if (R[3] > Rc) {
	    // here, outer and inner are furthest; typical case
	    foundBranch = 3;
	    d = outer; m = middle; t = inner;
	    Jd = 1; Jm = 3; Jt = 2;
	}
	if (firstRecalTrk) {
	    System.out.println("  foundBranch = "+foundBranch);
	}

	if (foundBranch!=0) {
	    xd = d.x; double xm = m.x; double xt = t.x;
	    yd = d.y; double ym = m.y; double yt = t.y;
	    zd = d.z; double zm = m.z; double zt = t.z;

	    double zDiff = 0.5*(Math.abs(zd-zt) - Math.abs(zm-(zd+zt)));
	    if (firstRecalTrk) System.out.println("  zDiff = "+zDiff);
	    if (Hpnts) Histogram.find("zDiff").fill(zDiff);

	    // Get the P,Q,R terms and then the triangle area
	    double Pot = xt - xd;
	    double Qot = yt - yd;
	    double Pom = xm - xd;
	    double Qom = ym - yd;

	    double Rot = Pot*Pot + Qot*Qot;
	    double Rom = Pom*Pom + Qom*Qom;
	    double Aire = -(Pot*Qom - Pom*Qot)/2.0;
	    if (Hpnts) Histogram.find("Aire").fill(Aire);
	    if (firstRecalTrk) System.out.println("  Aire = "+Aire);
	    if (Aire == 0.) Aire = 1.0E-06;

	    // Calculate the center of circle relative to point Id
	    xc = -0.5*(Rom*Qot - Rot*Qom)/Aire;
	    yc = -0.5*(Rot*Pom - Rom*Pot)/Aire;
	    // Get the gamma vector that is used for the azimuth calculation
	    gamma1 = 2.0*xc*Aire;
	    gamma2 = 2.0*yc*Aire;
	    double gammam = Math.sqrt(gamma1*gamma1 + gamma2*gamma2);
	    // Now get the absolute center of the circle
	    xxc = xd - 0.5*xc;
	    yyc = yd - 0.5*yc;
	    double Dd = 0.5*(xc*xc + yc*yc);
	    // and the curvature
	    curv = -4.0 * Aire/gammam;

	    // Calculate the arc lengths from point D
	    double Psm = Dd - R[Jt];
	    double Pvm = xc*Q[Jt] - yc*P[Jt];
	    double Pst = Dd - R[Jm];
	    double Pvt = -xc*Q[Jm] + yc*P[Jm];
/*	    double Pst = xxc*(xxc - xt) + yyc*(yyc - yt);
	    double Pvt = xt*yyc - yt*xxc; */
	    double sm = Math.atan2(Pvm,Psm);
	    double st = Math.atan2(Pvt,Pst);
	    // Calculate the quantities needed for tz
	    double Psd = Dd -xd*xc - yd*yc;
	    double Pvd = xc*yd - yc*xd;
/*	    double Asm = Math.atan2(Pvd,Psd) + (int)(sm/Math.PI)*(2.*Math.PI);
	    double Asp = Math.atan2(Pvt,Pst) + (int)(sp/Math.PI)*(2.*Math.PI);
	    double Ast = Asp - Asm;
	    tz = (zt - zd)/Ast;
*/
	    // Calculate tan(dip)*radius from points T and D
	    tz = (zt - zd)/st;
	    // Calculate deviation of point M in z            
	    double dz0 = zm - (zd + tz*sm);
	    if (firstRecalTrk) System.out.println("  dz0 = "+dz0);
	    if (Hpnts) Histogram.find("dz0").fill(dz0);

	    double U = (xd*gamma1 + yd*gamma2)
		/Math.sqrt(gamma1*gamma1+gamma2*gamma2);
	    double d0 = (2.0*U+curv*(xd*xd+ yd*yd)) /
		(2.0+0.5*curv*(2.0*U+curv*(xd*xd+ yd*yd)));
	    double arcor = Math.atan2((-xc*yyc+yc*xxc),
				      (-xc*xxc-yc*yyc));

	    double V = (xd*gamma2 - yd*gamma1)
		/Math.sqrt(gamma1*gamma1+gamma2*gamma2);
	    double psi = Math.atan2(curv*V,(1.0+curv*U));
	    double av1 = Math.sin(psi)/curv;
	    double cosPsi = Math.cos(psi);
	    double av2;
	    if (cosPsi < 0) av2 = (1.0 - cosPsi)/curv;
	    else av2 = av1*Math.sin(psi)/(1.0 + cosPsi);
	    double beta1 = (curv*(av1*gamma2-av2*gamma1) 
			    + gamma1)/Math.sqrt(gamma1*gamma1+gamma2*gamma2);
	    double beta2 = (-curv*(av1*gamma1+av2*gamma2) 
			    + gamma2)/Math.sqrt(gamma1*gamma1+gamma2*gamma2);

	    double phi0 = Math.atan2(-beta1,beta2);
	    double z0 = zd+tz*arcor;
	    double s = tz*curv;

	    // System.out.println(" d0,curv = "+d0+","+curv);
	    // System.out.println(" tz = "+tz);
	    // System.out.println(" z0 = "+z0);
	    // System.out.println(" phi0 = "+phi0);

	    if (Htrks) {
		Histogram.find("d0").fill(d0);
		Histogram.find("phi0").fill(phi0);
		Histogram.find("curv").fill(curv);
		Histogram.find("z0").fill(z0);
		Histogram.find("s").fill(s);
	    }
	    if (Hdiff) {
		Histogram.find("diff. d0").fill(d0-parameters.d0);
		Histogram.find("diff. phi0").fill(phi0-parameters.phi0);
		Histogram.find("diff. omega").fill(curv-parameters.omega);
		Histogram.find("diff. z0").fill(z0-parameters.z0);
		Histogram.find("diff. s").fill(s-parameters.s);
	    }

	    // Store recalculated track parameters
	    parameters.put_d0(d0);
	    parameters.put_phi0(phi0);
	    parameters.put_curv(curv);
	    parameters.put_z0(z0);
	    parameters.put_s(s);

	    firstRecalTrk = false;
	}
	// for (int i = 1; i <= nhits; i++) {
	recalculated = true;
//	calculateChiSq(specifications);
    }

    public double calculateChiSq(Specifications specifications)
    {
	double rPhiRms = specifications.getRPhiResolution(), zRms = specifications.getZResolution();
	double rPhiRoad = specifications.getNSigmaTracking()*rPhiRms, zRoad = specifications.getNSigmaTracking()*zRms;

	if (firstCalcChiSq) {
	    System.out.println(ClassName+"calculateChiSq: ");
	    System.out.println("  Tracker specifications:");
	    System.out.println(" \t r-phi,z resolutions = "+specifications.getRPhiResolution()+","+specifications.getZResolution());
	    System.out.println(" \t  # sigma tracking   = "+specifications.getNSigmaTracking());
	    System.out.println("  use \n"+
			       " \t  rPhi,zRoad = "+df.format(rPhiRoad)+","+df.format(zRoad));
	    firstCalc1 = true;
	    firstCalc2 = true;
	}
	//if (!recalculated) recalculateTrack(specifications);
	ChiSq = 0.;
	npteff = 0;

	int npoints = 0;

	// Set the number of points on the side of interest to zero
	int npco = 0;
	double pisi = (2.0*Math.PI)*si; // signed 2 pi

	double HalfLength = specifications.getHalfLength();
	swimmer = new HelicalSwimmer(parameters,specifications.getBField(),
				     specifications.getOuterRadius(),HalfLength);

	TrkHitOnTrk hot;
	for (int i=0;i<Math.min(hits.nhits,5);i++) {
	// for (int i=0;i<hits.nhits;i++) {
	    hot = hits.get(i);
	    if (hot.isActive) {
		npoints++;
	      /*
		// Get x and y differences from point D.
		double xt = hot.x - xd;
		double yt = hot.y - yd;
		// Calculate square of distance from point D.
		double Rdd = xt*xt + yt*yt;
		// Calculate approximate deviation from circle.
		double pc = xt*xc + yt*yc;
		double al = (Rdd+pc)/Math.sqrt(xc*xc+yc*yc);

		// Continue only if this deviation isn't too large
		if (Hhits && HTracks<=MaxHistPnts) Histogram.find("al").fill(Math.abs(al));
		//boolean largeDev = Math.abs(al) > Dmasu;
		//boolean largeDev = Math.abs(al) > 10.;
		boolean largeDev = Math.abs(al) > rPhiRoad;
		if (!largeDev) {
		    if (Hhits && HTracks<=MaxHistPnts) Histogram.find("al cand.").fill(Math.abs(al));
		  /*
                    //cross product of CD and CP
		    double ps = xc*yt - yc*xt;
		    //turning angle
		    double Dd = 0.5*(xc*xc+yc*yc);
		    double as = Math.atan2(ps,(Dd+pc));
		    //DC x OD = OC x OD
		    double pss = xd*yc - yd*xc;

		    // pow true means pipe on the way
		    boolean pow = (pss*si)>0.0;
		    // cote is true if the point is not on the same half circle
		    // as point t, with the origin defined by point d.
		    boolean cote = (ps*si)<0.0;
		    // OM x OC
		    boolean crspc = (hot.x*yyc - hot.y*xxc)>0.0;
		    boolean crsdc = (xd*yyc - yd*xxc)<0.0;
		    // cotec is true if the point is on the same side of OC as D
                    boolean cotec = !(crsdc^crspc);
		    if (firstCalc1) System.out.println("  pow,cote,crspc,crsdc,cotec = "+
						      pow+" "+cote+" "+crspc+" "+crsdc+" "+cotec);

		    // IF THE CIRCLE DOESN'T ENCOUNTER ANY BOUNDARY, WE MAKE TWO TURNS
		    // consider all but opposite side hits of tracks 
		    // that hit both walls
                    double as2, s, Zz;
		    if (firstCalc1) {
			System.out.println("  OR !outt,outp,!cotec = "+!outt+" "+outp+" "+!cotec);
			firstCalc1 = false;
		    }
                    if((!outt) || outp || (!cotec)) {   // do points on tracks that miss both walls
		      if(outp && (!outt)) {
		        boolean xorTemp3,invXorTemp3; 
                        xorTemp3 = outp^pow;
                        if (cote && !xorTemp3) as = as + pisi;
                        if ((!cote) && (xorTemp3)) as = as - pisi;
                        s=as;  // calculate arc length
                      } else {
		        if(cotec) {  // points on opposite side of track
			  if (cote) as2 = as + pisi;
			  else as2 = as - pisi;
			  s = as2;
		        } else {     // points on same side
		          s = as;
		        }
		      }
                      Zz = hot.z - (zd + s*tz);
		      if (firstCalc2) System.out.println("  Zz = "+Zz);
		      firstCalc2 = false;
		      if (Hpnts && nCalc<=100) Histogram.find("Zz").fill(Zz);
                      // boolean smallZRes = Math.abs(Zz) < Ddzax;
                      boolean smallZRes = Math.abs(Zz) < 1.0;

                      if (smallZRes) {
			  if (Hpnts && nCalc<=100) Histogram.find("Zz pass").fill(Zz);
		  */

            // We have a good point - increment point counter.
	    npteff = npteff + 1;

	    // Store point number and ...
	    // int ip = ipnt[0];
	    // ipnt[ip] = i;
	    // Sang[i] = -s*si;
	    // hot.putSAng(-s*si);

	    // Swim track to point and check agreement
	    double r = Math.sqrt(hot.x*hot.x + hot.y*hot.y);
	    double phi = Math.atan2(hot.y,hot.x);
	    swimmer.swim(r,HalfLength); double[] point = swimmer.getIntersect();
	    if (i<5) System.out.println(" hot x,y,z = "+df.format(hot.x)+","+df.format(hot.y)+","+df.format(hot.z)+
					" track = "+df.format(point[0])+","+df.format(point[1])+","+df.format(point[2]));
	    double r1 = Math.sqrt(point[0]*point[0]+point[1]*point[1]);
	    double phi1 = Math.atan2(point[1],point[0]);
	    if (Hdev && HTracks<=MaxHistPnts) {
		//Histogram.find("delta R").fill(r-r1);
		// if (i<5||i>60) System.out.println(" hot x,y = "+hot.x+","+hot.y+" track = "+point[0]+","+point[1]);
		if (Math.abs(r-r1)<0.0001) {
		    // if (i!=IInner && i!=IMiddle && i!=IOuter) {
		        Histogram.find("delta Z").fill(hot.z-point[2]);
			double deltaRPhi = r*(phi-phi1);
			Histogram.find("delta rphi").fill(deltaRPhi);
		    // }
		}
	    }

	/*
	    // Get a crude chi-squared for the point
	    // double Dmas = 0.009;
	    // double Dzas = 0.020;
	    // double Dmas = 1.0;
	    // double Dzas = 1.0;
	    // double chis = (al/Dmas)*(al/Dmas) + (Zz/Dzas)*(Zz/Dzas);
	    // ChisN[i] = (al/Dmasu)*(al/Dmasu) + (Zz/Ddzax)*(Zz/Ddzax);
	    double chiSq = (al/rPhiRms)*(al/rPhiRms);
	    // double chiSq = (al/rPhiRms)*(al/rPhiRms) + (Zz/zRms)*(Zz/zRms);
	    // if (Hpnts && nCalc<=1000) Histogram.find("Chis").fill(chis);
	    if (HchiSq && HTracks<=MaxHistPnts) {
		Histogram.find("chiSq").fill(chiSq);
		Histogram.find("rms").fill(Math.sqrt(chiSq));
		// if (i!=IInner && i!=IMiddle && i!=IOuter) {
		    // double resid = Math.sqrt(al*al+Zz*Zz);
		    Histogram.find("ResidXY").fill(al);
		    Histogram.find("ResidZ").fill(Zz);
		    // Histogram.find("Resid").fill(resid);
		// }
	    }
	    // hot.putChiSq(chis);

	    ChiSq = ChiSq + chiSq;
	/*
	    if (!cote) npco++;

		      } //end the small Z deviation

		    } else {
			if (firstFailBW) {
			    System.out.println("  Track failing both walls check");
			    firstFailBW = false; }
		    } //end miss both walls

		  }//end the largeDev check
	*/
	    } // active
        } // nhits
/*
	// Extrapolate track to EM and HAD Calorimeters
	doExtrapolations(parameters,specifications);

	if (npteff<5) {
	    System.out.println(ClassName+"calcChiSq: Track candidate w/ only "+npteff+" eff. points.");
	    System.out.println("  nhits = "+hits.nhits);
	    FailNpts = true;
	    Candidate = false;
	}
/*
            double SangId=0.;
	    for (int I1=1; I1<=itra[it][0]; I1++) 
	      if (itra[it][I1] == Id) SangId = Sang[I1];
	    for (int I2=1; I2<=itra[it][0]; I2++) {
	      path[it][I2-1] = -arcor/curv + 
	        (Sang[I2] - SangId)/Math.abs(curv);
	    }
*/
/*
	if (Hstd) {
	    Histogram.find("ChiSq").fill(ChiSq);
	    Histogram.find("Npteff").fill(npteff);
	    if (npteff>0) {
		double ChiSqN = ChiSq/npteff;
		Histogram.find("ChiSqN").fill(ChiSqN);
	    }
	}
*/
	firstCalcChiSq = false;
	return ChiSq;
    }


    /** Extrapolate track to outer detectors. */
    public void doExtrapolations(TrackParameters tp, Specifications specifications)
    {
	extrapolations = new TrackExtrapolations();

	HelicalSwimmer swimmer;

	// Swim track to EM calorimeter
	if (firstExtrapolation) {
	    System.out.println(ClassName+"doExtrapolations: ");
	    System.out.println("  Calorimeter specifications:");
	    System.out.println(" \t EM  Inner,Outer radii = "+specifications.get_EM_InnerRadius()+","+specifications.get_EM_OuterRadius());
	    System.out.println(" \t HAD             radii = "+specifications.get_HAD_InnerRadius()+","+specifications.get_HAD_OuterRadius());
	}
	swimmer = new HelicalSwimmer(parameters,specifications.getBField(),
				     specifications.get_EM_InnerRadius(),specifications.get_EM_InnerEndPlane());
	swimmer.swim(specifications.get_EM_InnerRadius(),specifications.get_EM_InnerEndPlane());
	extrapolations.put_EM_Extrapolation1(swimmer.getIntersect());

	swimmer = new HelicalSwimmer(parameters,specifications.getBField(),
				     specifications.get_EM_OuterRadius(),specifications.get_EM_OuterEndPlane());
	swimmer.swim(specifications.get_EM_OuterRadius(),specifications.get_EM_OuterEndPlane());
	extrapolations.put_EM_Extrapolation2(swimmer.getIntersect());

	// Swim track to HAD calorimeter
	swimmer = new HelicalSwimmer(parameters,specifications.getBField(),
				     specifications.get_HAD_InnerRadius(),specifications.get_HAD_InnerEndPlane());
	swimmer.swim(specifications.get_HAD_InnerRadius(),specifications.get_HAD_InnerEndPlane());
	extrapolations.put_HAD_Extrapolation1(swimmer.getIntersect());

	swimmer = new HelicalSwimmer(parameters,specifications.getBField(),
				     specifications.get_HAD_OuterRadius(),specifications.get_HAD_OuterEndPlane());
	swimmer.swim(specifications.get_HAD_OuterRadius(),specifications.get_HAD_OuterEndPlane());
	extrapolations.put_HAD_Extrapolation2(swimmer.getIntersect());

	firstExtrapolation = false;
    }

    public void print()
    {
	System.out.println(ClassName+" Track candidate #"+trkNum);
	System.out.println("  parameters: \n"+
			   "    d0     = "+df.format(parameters.d0) +  "\t"+"   phi0 = "+df.format(parameters.phi0)+"\n"+
			   "    omega  = "+df.format(parameters.omega)+"\t"+"   z0   = "+df.format(parameters.z0)+"\n"+
			   "    s      = "+df.format(parameters.s));
    }
    public void printPoints(int npts)
    {
	System.out.println(ClassName+"printPoints: Track candidate #"+trkNum);
	int nhits = hits.getNHits();
	// int nhits = hits.getNActive();
	TrkHitOnTrk hot;
	for (int i=0;i<Math.min(nhits,npts);i++) {
	    hot = hits.get(i);
	    hot.print(i);
	}
	if (nhits>npts) {
	    System.out.println("  ...");
	    hot = hits.get(nhits-1);
	    hot.print(nhits-1);
	}
    }

    static DecimalFormat df = new DecimalFormat();

    static {
	df.setMaximumFractionDigits(3);
    }


    static String ClassName = "TrackCandidate::"; static String text;

    static boolean firstTrkCand   = true;
    static boolean firstRecalTrk  = true;
    static boolean firstExtrapolation = true;
    static boolean firstCalcChiSq = true;

    static boolean firstUseOrigin = true;
    static boolean firstFailBW    = false;
    static boolean firstFailEffPts = false;

    static boolean firstCalc1=false,firstCalc2=false;

    // Histogramming
    //String defaultFolder = "/Tracking/Add";
    static int HTracks = 0;
    static int MaxHistPnts = 100, MaxHistTrks = 1000;
    static boolean Hist;
    static boolean Hstd   = false;

    static boolean Htrks  = false;
    static boolean Hhits  = false;
    static boolean Hpnts  = false;
    static boolean HchiSq = false;
    static boolean Hdiff  = false;

    static boolean Hdev   = false;


    boolean recalculated=false;
    TrkHitOnTrk d,m,t;
    double xd,yd,zd,xc,yc,xxc,yyc,si,tz;
    double gamma1,gamma2,curv;
    boolean outp,outt;
    boolean FailNpts=false;

    static HelicalSwimmer swimmer;
}
