/* TrackExtrapolations.java

 Created by  Mike Ronan  Dec '98   
 Modified by
   Mike Ronan    Feb '00    - Develop extrapolations to calorimeters

 */

package hep.lcd.recon.ztracking;

import java.io.Serializable;
import java.text.*;
import java.util.*;

public class TrackExtrapolations implements Serializable
{
    public double[] EMextrapolation1  = new double[3];
    public double[] EMextrapolation2  = new double[3];
    public double[] HADextrapolation1 = new double[3];
    public double[] HADextrapolation2 = new double[3];

    static boolean firstEMextrapolation1  = true;
    static boolean firstEMextrapolation2  = true;
    static boolean firstHADextrapolation1 = true;
    static boolean firstHADextrapolation2 = true;

    /** Used by TrackFitter's in creating a ReconstructedTrack. */
    public TrackExtrapolations()
    {
    }

    public double[] extrapolation = new double[3];

    /** Backward compatibility */
    public TrackExtrapolations(double[] extrapolation)
    {
	this(extrapolation[0],extrapolation[1],extrapolation[2]);
    }
    public TrackExtrapolations(double x, double y, double z)
    {
	extrapolation[0] = x;
	extrapolation[1] = y;
	extrapolation[2] = z;
    }

    public double[] get_EM_Extrapolation1() { return EMextrapolation1; }
    public double[] get_EM_Extrapolation2() { return EMextrapolation2; }
    public double[] get_EM_Extrapolation()  { return EMextrapolation1; }
    public double get_EM_X() { return EMextrapolation1[0]; }
    public double get_EM_Y() { return EMextrapolation1[1]; }
    public double get_EM_R() {
	return Math.sqrt(EMextrapolation1[0]*EMextrapolation1[0]+
			 EMextrapolation1[1]*EMextrapolation1[1]); }
    public double get_EM_Z() { return EMextrapolation1[2]; }

    public double[] get_HAD_Extrapolation1() { return HADextrapolation1; }
    public double[] get_HAD_Extrapolation2() { return HADextrapolation2; }
    public double[] get_HAD_Extrapolation()  { return HADextrapolation1; }
    public double get_HAD_X() { return HADextrapolation1[0]; }
    public double get_HAD_Y() { return HADextrapolation1[1]; }
    public double get_HAD_R() {
	return Math.sqrt(HADextrapolation1[0]*HADextrapolation1[0]+
			 HADextrapolation1[1]*HADextrapolation1[1]); }
    public double get_HAD_Z() { return HADextrapolation1[2]; }

    public void put_EM_Extrapolation1(double[] extrap)
    {
	put_EM_Extrapolation1(extrap[0],extrap[1],extrap[2]);
    }
    public void put_EM_Extrapolation1(double x, double y, double z)
    {
	EMextrapolation1[0] = x;
	EMextrapolation1[1] = y;
	EMextrapolation1[2] = z;
	if (firstEMextrapolation1)
	    System.out.println(" Track extrapolates to inside of EM calorimeter at "+
			       df.format(x)+","+df.format(y)+","+df.format(z));
	firstEMextrapolation1 = false;
    }
    public void put_EM_Extrapolation2(double[] extrap)
    {
	put_EM_Extrapolation2(extrap[0],extrap[1],extrap[2]);
    }
    public void put_EM_Extrapolation2(double x, double y, double z)
    {
	EMextrapolation2[0] = x;
	EMextrapolation2[1] = y;
	EMextrapolation2[2] = z;
	if (firstEMextrapolation2)
	    System.out.println(" Track extrapolates to outside of EM calorimeter at "+
			       df.format(x)+","+df.format(y)+","+df.format(z));
	firstEMextrapolation2 = false;
    }

    public void put_HAD_Extrapolation1(double[] extrap)
    {
	put_HAD_Extrapolation1(extrap[0],extrap[1],extrap[2]);
    }
    public void put_HAD_Extrapolation1(double x, double y, double z)
    {
	HADextrapolation1[0] = x;
	HADextrapolation1[1] = y;
	HADextrapolation1[2] = z;
	if (firstHADextrapolation1)
	    System.out.println(" Track extrapolates to inside of HAD calorimeter at "+
			       df.format(x)+","+df.format(y)+","+df.format(z));
	firstHADextrapolation1 = false;
    }
    public void put_HAD_Extrapolation2(double[] extrap)
    {
	put_HAD_Extrapolation2(extrap[0],extrap[1],extrap[2]);
    }
    public void put_HAD_Extrapolation2(double x, double y, double z)
    {
	HADextrapolation2[0] = x;
	HADextrapolation2[1] = y;
	HADextrapolation2[2] = z;
	if (firstHADextrapolation2)
	    System.out.println(" Track extrapolates to outside of HAD calorimeter at "+
			       df.format(x)+","+df.format(y)+","+df.format(z));
	firstHADextrapolation2 = false;
    }

    public boolean reachesRadius(double radius)
    {
	boolean does = true;
	if (Math.sqrt(EMextrapolation1[0]*EMextrapolation1[0]+
		      EMextrapolation1[1]*EMextrapolation1[1])<radius) does = false;
	return does;
    }
    public boolean exitsEnd(double z)
    {
	boolean does=false;
	if (Math.abs(EMextrapolation1[2])>=z) does=true;
	return does;
    }

    static DecimalFormat df = new DecimalFormat();

    static {
	df.setMaximumFractionDigits(3);
    }

    String ClassName = "TrackExtrapolations::";
}
