/* TrackFitter.java

		Defines interface to track fitters.

 Created by  Mike Ronan  Dec '98
 Modified by

 */

package hep.lcd.recon.ztracking;

import hep.lcd.event.*;


/**  Track fitter interface. */
public interface TrackFitter
{
    /** Setup Fitter parameters and initiate callback to set fitting strategy. */
    public void setupFitter(Tracker tracker, VertexDetector vxd);

    public void setTracker(Tracker tpc);
    public void setVXD(VertexDetector vxd);
    public void useVXD(boolean use);
    public void useTracker(boolean use);

    //public void setupEvent(LCDEvent e); 

    /** Fit tracks. */
    // public ReconstructedTrack fitTrack(double[][] points, double[] trackPar);
    // public ReconstructedTrack fitTrack(TrkHotList hits, double[] trackPar);
    // public ReconstructedTrack fitTrack(TrkCandidate tc);
    public void fit(int itrk, VertexDetector vxd, Tracker tracker);
    public void fit(int itrk);

    /** Returns status of track fitting. */
    public boolean fitSuccess();
    /** Returns fitted track parameters. */
    public TrackParameters getFittedParameters();
    /** Get overall chi-squared for the track. */
    public double getChi2();
    /** Get number of degrees of freedom for the track. */
    public double getNDF();
    /** Get track error matrix. */
    public double[][] getErrorMatrix();

    /** Turns on Track fitter production histograms. */
    public void setHprod();
    /** Turns on Track fitter test histograms. */
    public void setHtest();
    /** Turns on Track fitter development histograms. */
    public void setHdev();

    /** Turns off all histogram packages */
    public void setNoHist();
}
