/* CombinedPat2.java

		FIND TRACKS FROM THREE POINTS

    CombinedPat2 is derived from TPC only pattern recognition TPCPat2.

    TPCPat2 was derived from PAT2, which was written by Henri Videau
    for PEP-TPC tracks.  The main coding and logic remains from the
    original version, though many changes were made over the years by
    Gerry Lynch and Orin Dahl.

    TPCPat2 also inherits from BBPAT2, a derivative made by Gerry Lynch
    for SVT track finding in BaBar, and the C++ version BaBarPat2 written by
    Natalia Kuznetsova. Some treatments of ambiguities, developed specifically
    for BaBar, may still remain in the code and should be reviewed.

 Created by  Mike Ronan  Nov '98   Use 3D points for LCD events within JAS framework to find charged tracks
                                   using BaBarPat2 algorithms and large TPC strategy.
 Modified by:
   Nick Sinev               Modified to include both VXD and TPC into track finding.
   Mike Ronan    May '01    Re-establish tracking object model

 */

package hep.lcd.recon.ztracking.combined;

import hep.lcd.recon.ztracking.*;

import hep.analysis.*;
import hep.physics.*;

import java.util.*;
import java.text.*;


/**   Finds tracks from 3D space points using various triplets of layers. Graded levels
  * of pattern recognition are employed to optimize tracking efficiency.
  */
public class CombinedPat2 extends TrackFinder
{
    static int MAXTracks  = 200;
    static int MAXTrkPnts = 250;

/*
    Specifications specifications;

    int plas[][][];           // points indexes arranged per layer and per azimuthal angle    
    double pazi[];
    double minAz = 0.;
    double maxAz = 0.;
    double Dzax = 0.;
    double Dmasu = 0.;
    double tdInnerR = 0.;
    double tdOuterR = 0.;
    int nPrintL = 0;
    int minVxHits = 0;
    int minTrHits = 2;
    int nVxLrs = 0;
    boolean vectorsOnly = false;
    double xyerv = 0.001;
    double zerv = 0.001;
    double xyert = 0.003;
    double zert = 0.003;
    double msctrm = 0.001;
    double minRad = 8.;
    int numberPtsRequired = 1000;
    private boolean trace = false;
    private int tracehc = 57;
    private boolean dotrace = false;
*/

    /**  Construct TPC Track Finder for given MAXTracks and MAXTrkPnts. */
    public CombinedPat2(boolean hist)
    {
   	super(MAXTracks,MAXTrkPnts,hist); 

	System.out.println(ClassName+"  version 1.1.8.5");

        Finder = "CombinedPat2";

        // plas = new int[MaxLayers][NAzBins][MaxPntBin+1];
        // pazi = new double[MaxPoints];
    }

    /** Invoked in setting up TrackFinder. Sets track finding strategy. */
    public void setupParameters()
    {
	if (debug) System.out.println(ClassName+"::setupParameters:  ");

	strategy = new CombinedTrackingStrategy();

	if (getTracker() == null) System.out.println("TRACKER IS NOT set!"); 
	else {
	    specifications = (Specifications) getTracker().getSpecifications();
	    int detectorType = specifications.getDetectorType();
	    // tdInnerR = specifications.getInnerRadius();
	    // tdOuterR = specifications.getOuterRadius();

	    int NLevels = strategy.getNLevels(detectorType);
	    double[] minRadius = strategy.getMinRadius(detectorType);
	    int NPatterns = strategy.getNPatterns(detectorType);
	    int[][] triplets = strategy.getTriplets(detectorType);
	    int[] nPtsReq = strategy.getNPtsReq(detectorType);     // Number of points required for each pattern

	    rTol = strategy.getRTol(detectorType);  // max. distance from origin in xy
	    zTol = strategy.getZTol(detectorType);  // max distance from origin in z

	    if (debug) {
	      System.out.println(" DetectorType = "+detectorType);
	      System.out.println("  NLevels   = "+NLevels);
	      System.out.println("  minRadius = "+minRadius[0]+", ...");
	      System.out.println("  NPatterns = "+NPatterns);
	      System.out.println("  triplets  = "+triplets[0][0]+","+triplets[0][1]+","+triplets[0][2]+"; ...");
	      System.out.println("  nPtsReq   = "+nPtsReq[0]+", ...");
	      System.out.println("  r,zTol    = "+rTol+","+zTol);
	    }

	    tooMany = 20000000;  // Max. # combinations 2 10^7 
	    //dmas = 0.05;         // phi tolerance 
	    // dmas = 0.1;          // phi tolerance 
	    dmas = 0.5;         // phi tolerance 
	    //Dzas = 2.0;          // z tolerance (cm) (less than 0.3 cuts finding efficiency)
	    // Dzas = 1.0;          // z tolerance (cm)
	    Dzas = 0.5;          // z tolerance (cm)

	    // xyert = 0.03;
	    // zert = 0.4;
	    // msctrm = 0.006;
	    rPipe = 2.0;          // beam pipe radius
	    rOut  = 200.0;        // solenoid radius
	    // minVxHits = 0;
	    // minTrHits = 15;
	    // vectorsOnly = false;

	    if (detectorType == 2) {
		dmas = 0.05;          // phi tolerance (radians)
		Dzas = 0.2;           // z tolerance (cm)
	        // dmas = 0.0;          // phi tolerance (radians)
		// Dzas = 0.025;           // z tolerance (cm)
		// xyerv = 0.0;
		// zerv  = 0.0;
		// xyert = 0.01;
		// zert = 0.01;
		// msctrm = 0.01;
		rPipe = 1.0;          // beam pipe radius
		rOut  = 100.0;        // solenoid radius
		// minVxHits = 1;
		// minTrHits = 2;
		// vectorsOnly = true;
	    }
	    Ddzax = Dzas;

	    // Setup TrackFinder pattern recognition strategy.
	    setupStrategy(NLevels,minRadius,NPatterns,triplets,nPtsReq,rTol,zTol);

	    if (debug)
	      System.out.println(" For now  rPipe,rOut = "+rPipe+", "+rOut); 
	}
    }


    String ClassName = "CombinedPat2::";

}
