/* TrackReconstruction.java

		Find and reconstruct tracks.

 Created by  Mike Ronan  Dec '98   Picks up LCD events within JAS framework and
                                   reconstructs tracks using 3D pattern recognition software.
 Modified by

 */

package hep.lcd.recon.ztracking.tpc;

import hep.lcd.recon.ztracking.*;


public class TrackReconstruction extends hep.lcd.util.driver.Driver
{
    static boolean Hist = false;

    /**
     * 
     */
    public TrackReconstruction()
    {
	this(Hist);
    }

    public TrackReconstruction(boolean hist)
    {
	System.out.println("");
	String text = "TrackReconstruction  version 1.1.9"+"\n"+
	    "  Running hep.lcd version "+hep.lcd.Version.getVersion();
	System.out.println(text); //if (!XReconstruction.getBatchMode()) out.println(text);
	System.out.println(" Track reconstruction using TPC and CCD detectors - ");
	System.out.println(" Track fitting using SLD track fitter - ");

	// Finds tracks in main tracker.
	tracker = new hep.lcd.recon.ztracking.tpc.TPCTracker(hist);

	// Assigns Vertex Detector hits to tracks found in main tracker.
	vertexDetector = new hep.lcd.recon.ztracking.ccd.CCDVertexDetector(tracker,hist);

	// Set up track fitting
	fitter = new hep.lcd.recon.ztracking.fitters.sld.SLDTrackFitter(hist);

	if (fitter!=null) fitTracks = new FitTracks(tracker,vertexDetector,fitter,hist);

	// Adds reconstructed tracks to event
	addTracks = new AddReconstructedTracks(tracker,vertexDetector,hist);


	// Special histogramming
        // ---------------------
	if (hist) {
	  boolean test = false;
	  boolean dev  = false;
	  if (test) {
            tracker.setHtest();  // Turn on Tracker test histogram package
	    tracker.getTrackFinder().setHtest();  // Track finder test histogram package
            vertexDetector.setHtest();  // Turn on Vertex Detector test histogram package
	    if (fitTracks!=null) {
	      fitTracks.setHtest();  // Turn on Track fitting histograms
	      fitter.setHtest(); // Turn on Track fitter test histogram package
	    }
            addTracks.setHtest();// ...
	  } else if (dev) {
            tracker.setHdev();   // Turn on Tracker development histogram package
            vertexDetector.setHdev();   // Turn on Vertex Detector development histogram package
	    if (fitTracks!=null) {
	      fitTracks.setHdev();// Turn on Track fitting histograms
	      fitter.setHdev();   // Turn on Track fitter development histogram package
	    }
            addTracks.setHdev(); // ...
	  } else {
            tracker.setHprod();  // Turn on Tracker production histogram package
            vertexDetector.setHprod();  // Turn on Vertex Detector production histogram package
	    if (fitTracks!=null) {
	      fitTracks.setHprod();// Turn on Track fitting histograms
	      fitter.setHprod();   // Turn on Track fitter production histogram package
	    }
            addTracks.setHprod();// ...
	  }
	}


	// Add processes
        // -------------
	add(tracker);
	add(vertexDetector);
        if (fitTracks!=null) add(fitTracks);

	add(addTracks);
    }

    /**
     * Used for standalone reconstruction
     */
    public static void main(String[] argv)
    {
        String JobName = "TPCTrackReconstruction";
        System.err.println("Running "+JobName+" in Batch mode");
	try {
	    int nargs = 0;
	    hep.analysis.Job job = new hep.analysis.Job(JobName);
	    String fileName = argv[nargs]; nargs++;
	    int nEvents = Integer.parseInt(argv[nargs]); nargs++;
	    if (nEvents<0) nEvents = Math.abs(nEvents); else nEvents *= 1000;

	    System.err.println("Create a "+JobName+" job, analyze "+nEvents+" events and save histograms.");
	    job.setEventSource(new hep.lcd.io.jas.LCDEventSource(fileName));
	    job.addEventAnalyzer(new TrackReconstruction());
	    job.go(nEvents);
	    job.save(); // Save histograms
	    System.err.println("Histograms saved in "+JobName+".javahist.");
	}
	catch (jas.jds.module.ModuleException x) { x.printStackTrace(); }
	catch (Exception x) {
	    System.err.println("Usage: jre "+JobName+" <fileName> <# evts (k)>");
	}
    }

    /**  The following track reconstruction processes are accessible to derived classes:
     * tracker, vertexDetector, fitter, fitTracks and addTracks.
     */
    protected Tracker tracker;
    protected VertexDetector vertexDetector;
    protected TrackFitter fitter;
    protected FitTracks fitTracks;
    protected AddReconstructedTracks addTracks;
}
